package sun.misc;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

public class HexDumpEncoder extends CharacterEncoder
{
    private int offset;
    private int thisLineLength;
    private int currentByte;
    private byte[] thisLine;
    
    public HexDumpEncoder() {
        this.thisLine = new byte[16];
    }
    
    static void hexDigit(PrintStream printStream, byte b) {
        int n = (char)(b >> 4 & 0xF);
        int b2;
        if (n > '\t') {
            b2 = (char)(n - 10 + 65);
        }
        else {
            b2 = (char)(n + 48);
        }
        printStream.write(b2);
        int n2 = (char)(b & 0xF);
        int b3;
        if (n2 > '\t') {
            b3 = (char)(n2 - 10 + 65);
        }
        else {
            b3 = (char)(n2 + 48);
        }
        printStream.write(b3);
    }
    
    @Override
    protected int bytesPerAtom() {
        return 1;
    }
    
    @Override
    protected int bytesPerLine() {
        return 16;
    }
    
    @Override
    protected void encodeBufferPrefix(OutputStream outputStream) throws IOException {
        this.offset = 0;
        super.encodeBufferPrefix(outputStream);
    }
    
    @Override
    protected void encodeLinePrefix(OutputStream outputStream, int thisLineLength) throws IOException {
        hexDigit(this.pStream, (byte)(this.offset >>> 8 & 0xFF));
        hexDigit(this.pStream, (byte)(this.offset & 0xFF));
        this.pStream.print(": ");
        this.currentByte = 0;
        this.thisLineLength = thisLineLength;
    }
    
    @Override
    protected void encodeAtom(OutputStream outputStream, byte[] array, int n, int n2) throws IOException {
        this.thisLine[this.currentByte] = array[n];
        hexDigit(this.pStream, array[n]);
        this.pStream.print(" ");
        ++this.currentByte;
        if (this.currentByte == 8) {
            this.pStream.print("  ");
        }
    }
    
    @Override
    protected void encodeLineSuffix(OutputStream outputStream) throws IOException {
        if (this.thisLineLength < 16) {
            for (int i = this.thisLineLength; i < 16; ++i) {
                this.pStream.print("   ");
                if (i == 7) {
                    this.pStream.print("  ");
                }
            }
        }
        this.pStream.print(" ");
        for (int j = 0; j < this.thisLineLength; ++j) {
            if (this.thisLine[j] < 32 || this.thisLine[j] > 122) {
                this.pStream.print(".");
            }
            else {
                this.pStream.write((int)this.thisLine[j]);
            }
        }
        this.pStream.println();
        this.offset += this.thisLineLength;
    }
}
